@@ -1,19 +1,20 @@
 using System;
 using System.Collections;
-using System.Collections.Specialized;
+using System.Collections.Generic;
+using System.Diagnostics;
 using System.Runtime.InteropServices;
 using System.Reflection;
 using System.Text;
+using System.Collections.Generic;
 
 namespace Python.Runtime
 {
-    //=======================================================================
-    // This file defines objects to support binary interop with the Python
-    // runtime. Generally, the definitions here need to be kept up to date
-    // when moving to new Python versions.
-    //=======================================================================
-
-    [Serializable()]
+    /// <summary>
+    /// This file defines objects to support binary interop with the Python
+    /// runtime. Generally, the definitions here need to be kept up to date
+    /// when moving to new Python versions.
+    /// </summary>
+    [Serializable]
     [AttributeUsage(AttributeTargets.All)]
     public class DocStringAttribute : Attribute
     {
@@ -31,7 +32,7 @@ namespace Python.Runtime
         private string docStr;
     }
 
-    [Serializable()]
+    [Serializable]
     [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
     internal class PythonMethodAttribute : Attribute
     {
@@ -40,7 +41,7 @@ namespace Python.Runtime
         }
     }
 
-    [Serializable()]
+    [Serializable]
     [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
     internal class ModuleFunctionAttribute : Attribute
     {
@@ -49,7 +50,7 @@ namespace Python.Runtime
         }
     }
 
-    [Serializable()]
+    [Serializable]
     [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
     internal class ForbidPythonThreadsAttribute : Attribute
     {
@@ -59,7 +60,7 @@ namespace Python.Runtime
     }
 
 
-    [Serializable()]
+    [Serializable]
     [AttributeUsage(AttributeTargets.Property)]
     internal class ModulePropertyAttribute : Attribute
     {
@@ -68,67 +69,126 @@ namespace Python.Runtime
         }
     }
 
+    internal static class ManagedDataOffsets
+    {
+        static ManagedDataOffsets()
+        {
+            FieldInfo[] fi = typeof(ManagedDataOffsets).GetFields(BindingFlags.Static | BindingFlags.Public);
+            for (int i = 0; i < fi.Length; i++)
+            {
+                fi[i].SetValue(null, -(i * IntPtr.Size) - IntPtr.Size);
+            }
+
+            size = fi.Length * IntPtr.Size;
+        }
+
+        public static readonly int ob_data;
+        public static readonly int ob_dict;
+
+        private static int BaseOffset(IntPtr type)
+        {
+            Debug.Assert(type != IntPtr.Zero);
+            int typeSize = Marshal.ReadInt32(type, TypeOffset.tp_basicsize);
+            Debug.Assert(typeSize > 0 && typeSize <= ExceptionOffset.Size());
+            return typeSize;
+        }
+        public static int DataOffset(IntPtr type)
+        {
+            return BaseOffset(type) + ob_data;
+        }
+
+        public static int DictOffset(IntPtr type)
+        {
+            return BaseOffset(type) + ob_dict;
+        }
+
+        public static int Size { get { return size; } }
+
+        private static readonly int size;
+    }
+
+    internal static class OriginalObjectOffsets
+    {
+        static OriginalObjectOffsets()
+        {
+            int size = IntPtr.Size;
+            var n = 0; // Py_TRACE_REFS add two pointers to PyObject_HEAD
+#if PYTHON_WITH_PYDEBUG
+            _ob_next = 0;
+            _ob_prev = 1 * size;
+            n = 2;
+#endif
+            ob_refcnt = (n + 0) * size;
+            ob_type = (n + 1) * size;
+        }
+
+        public static int Size { get { return size; } }
+
+        private static readonly int size =
+#if PYTHON_WITH_PYDEBUG
+            4 * IntPtr.Size;
+#else
+            2 * IntPtr.Size;
+#endif
+
+#if PYTHON_WITH_PYDEBUG
+        public static int _ob_next;
+        public static int _ob_prev;
+#endif
+        public static int ob_refcnt;
+        public static int ob_type;
+    }
 
     [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
     internal class ObjectOffset
     {
         static ObjectOffset()
         {
-            int size = IntPtr.Size;
-            int n = 0; // Py_TRACE_REFS add two pointers to PyObject_HEAD
-#if Py_DEBUG
-            _ob_next = 0;
-            _ob_prev = 1 * size;
-            n = 2;
+#if PYTHON_WITH_PYDEBUG
+            _ob_next = OriginalObjectOffsets._ob_next;
+            _ob_prev = OriginalObjectOffsets._ob_prev;
 #endif
-            ob_refcnt = (n + 0)*size;
-            ob_type = (n + 1)*size;
-            ob_dict = (n + 2)*size;
-            ob_data = (n + 3)*size;
+            ob_refcnt = OriginalObjectOffsets.ob_refcnt;
+            ob_type = OriginalObjectOffsets.ob_type;
+
+            size = OriginalObjectOffsets.Size + ManagedDataOffsets.Size;
         }
 
-        public static int magic(IntPtr ob)
+        public static int magic(IntPtr type)
         {
-            if ((Runtime.PyObject_TypeCheck(ob, Exceptions.BaseException) ||
-                (Runtime.PyType_Check(ob) && Runtime.PyType_IsSubtype(ob, Exceptions.BaseException))))
-            {
-                return ExceptionOffset.ob_data;
-            }
-            return ob_data;
+            return ManagedDataOffsets.DataOffset(type);
         }
 
-        public static int DictOffset(IntPtr ob)
+        public static int TypeDictOffset(IntPtr type)
         {
-            if ((Runtime.PyObject_TypeCheck(ob, Exceptions.BaseException) ||
-                (Runtime.PyType_Check(ob) && Runtime.PyType_IsSubtype(ob, Exceptions.BaseException))))
-            {
-                return ExceptionOffset.ob_dict;
-            }
-            return ob_dict;
+            return ManagedDataOffsets.DictOffset(type);
         }
 
-        public static int Size(IntPtr ob)
+        public static int Size(IntPtr pyType)
         {
-            if ((Runtime.PyObject_TypeCheck(ob, Exceptions.BaseException) ||
-                (Runtime.PyType_Check(ob) && Runtime.PyType_IsSubtype(ob, Exceptions.BaseException))))
+            if (IsException(pyType))
             {
                 return ExceptionOffset.Size();
             }
-#if Py_DEBUG
-            return 6 * IntPtr.Size;
-#else
-            return 4 * IntPtr.Size;
-#endif
+
+            return size;
         }
 
-#if Py_DEBUG
+#if PYTHON_WITH_PYDEBUG
         public static int _ob_next;
         public static int _ob_prev;
 #endif
         public static int ob_refcnt;
         public static int ob_type;
-        private static int ob_dict;
-        private static int ob_data;
+        private static readonly int size;
+
+        private static bool IsException(IntPtr pyObject)
+        {
+            var type = Runtime.PyObject_TYPE(pyObject);
+            return Runtime.PyType_IsSameAsOrSubtype(type, ofType: Exceptions.BaseException)
+                || Runtime.PyType_IsSameAsOrSubtype(type, ofType: Runtime.PyTypeType)
+                && Runtime.PyType_IsSubtype(pyObject, Exceptions.BaseException);
+        }
     }
 
     [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
@@ -137,18 +197,16 @@ namespace Python.Runtime
         static ExceptionOffset()
         {
             Type type = typeof(ExceptionOffset);
-            FieldInfo[] fi = type.GetFields();
-            int size = IntPtr.Size;
+            FieldInfo[] fi = type.GetFields(BindingFlags.Static | BindingFlags.Public);
             for (int i = 0; i < fi.Length; i++)
             {
-                fi[i].SetValue(null, (i * size) + ObjectOffset.ob_type + size);
+                fi[i].SetValue(null, (i * IntPtr.Size) + OriginalObjectOffsets.Size);
             }
+
+            size = fi.Length * IntPtr.Size + OriginalObjectOffsets.Size + ManagedDataOffsets.Size;
         }
 
-        public static int Size()
-        {
-            return ob_data + IntPtr.Size;
-        }
+        public static int Size() { return size; }
 
         // PyException_HEAD
         // (start after PyObject_HEAD)
@@ -163,9 +221,7 @@ namespace Python.Runtime
         public static int suppress_context = 0;
 #endif
 
-        // extra c# data
-        public static int ob_dict;
-        public static int ob_data;
+        private static readonly int size;
     }
 
 
@@ -186,7 +242,7 @@ namespace Python.Runtime
 
         /* The *real* layout of a type object when allocated on the heap */
         //typedef struct _heaptypeobject {
-#if Py_DEBUG  // #ifdef Py_TRACE_REFS
+#if PYTHON_WITH_PYDEBUG
 /* _PyObject_HEAD_EXTRA defines pointers to support a doubly-linked list of all live heap objects. */
         public static int _ob_next = 0;
         public static int _ob_prev = 0;
@@ -196,7 +252,7 @@ namespace Python.Runtime
         public static int ob_refcnt = 0;
         public static int ob_type = 0;
         // }
-        public static int ob_size = 0;      /* Number of items in _VAR_iable part */
+        public static int ob_size = 0; /* Number of items in _VAR_iable part */
         // }
         public static int ob_shash = 0;
         public static int ob_sval = 0; /* start of data */
@@ -223,7 +279,8 @@ namespace Python.Runtime
             }
         }
 
-        public static IntPtr AllocModuleDef(string modulename) {
+        public static IntPtr AllocModuleDef(string modulename)
+        {
             byte[] ascii = Encoding.ASCII.GetBytes(modulename);
             int size = name + ascii.Length + 1;
             IntPtr ptr = Marshal.AllocHGlobal(size);
@@ -235,7 +292,8 @@ namespace Python.Runtime
             return ptr;
         }
 
-        public static void FreeModuleDef(IntPtr ptr) {
+        public static void FreeModuleDef(IntPtr ptr)
+        {
             Marshal.FreeHGlobal(ptr);
         }
 
@@ -268,8 +326,7 @@ namespace Python.Runtime
     /// </summary>
     internal class TypeFlags
     {
-#if PYTHON2
-        // these flags were removed in Python 3
+#if PYTHON2 // these flags were removed in Python 3
         public static int HaveGetCharBuffer = (1 << 0);
         public static int HaveSequenceIn = (1 << 1);
         public static int GC = 0;
@@ -307,8 +364,7 @@ namespace Python.Runtime
         public static int BaseExceptionSubclass = (1 << 30);
         public static int TypeSubclass = (1 << 31);
 
-#if PYTHON2
-        // Default flags for Python 2
+#if PYTHON2 // Default flags for Python 2
         public static int Default = (
             HaveGetCharBuffer |
             HaveSequenceIn |
@@ -318,13 +374,12 @@ namespace Python.Runtime
             HaveIter |
             HaveClass |
             HaveStacklessExtension |
-                HaveIndex |
-                0);
-#elif PYTHON3
-        // Default flags for Python 3
+            HaveIndex |
+            0);
+#elif PYTHON3 // Default flags for Python 3
         public static int Default = (
-                            HaveStacklessExtension |
-                            HaveVersionTag);
+            HaveStacklessExtension |
+            HaveVersionTag);
 #endif
     }
 
@@ -336,8 +391,8 @@ namespace Python.Runtime
 
     internal class Interop
     {
-        static ArrayList keepAlive;
-        static Hashtable pmap;
+        private static List<ThunkInfo> keepAlive;
+        private static Hashtable pmap;
 
         static Interop()
         {
@@ -353,8 +408,7 @@ namespace Python.Runtime
                 p[item.Name] = item;
             }
 
-            keepAlive = new ArrayList();
-            Marshal.AllocHGlobal(IntPtr.Size);
+            keepAlive = new List<ThunkInfo>();
             pmap = new Hashtable();
 
             pmap["tp_dealloc"] = p["DestructorFunc"];
@@ -451,7 +505,7 @@ namespace Python.Runtime
             return pmap[name] as Type;
         }
 
-        internal static IntPtr GetThunk(MethodInfo method, string funcType = null)
+        internal static ThunkInfo GetThunk(MethodInfo method, string funcType = null)
         {
             Type dt;
             if (funcType != null)
@@ -459,18 +513,15 @@ namespace Python.Runtime
             else
                 dt = GetPrototype(method.Name);
 
-            if (dt != null)
+            if (dt == null)
             {
-                IntPtr tmp = Marshal.AllocHGlobal(IntPtr.Size);
-                Delegate d = Delegate.CreateDelegate(dt, method);
-                Thunk cb = new Thunk(d);
-                Marshal.StructureToPtr(cb, tmp, false);
-                IntPtr fp = Marshal.ReadIntPtr(tmp, 0);
-                Marshal.FreeHGlobal(tmp);
-                keepAlive.Add(d);
-                return fp;
+                return ThunkInfo.Empty;
             }
-            return IntPtr.Zero;
+            Delegate d = Delegate.CreateDelegate(dt, method);
+            var info = new ThunkInfo(d);
+            // TODO: remove keepAlive when #958 merged, let the lifecycle of ThunkInfo transfer to caller.
+            keepAlive.Add(info);
+            return info;
         }
 
         [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
@@ -524,4 +575,22 @@ namespace Python.Runtime
             fn = d;
         }
     }
+
+    internal class ThunkInfo
+    {
+        public readonly Delegate Target;
+        public readonly IntPtr Address;
+
+        public static readonly ThunkInfo Empty = new ThunkInfo(null);
+
+        public ThunkInfo(Delegate target)
+        {
+            if (target == null)
+            {
+                return;
+            }
+            Target = target;
+            Address = Marshal.GetFunctionPointerForDelegate(target);
+        }
+    }
 }
